package org.jeecg.modules.okki.linkedin.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.SecurityUtils;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.system.vo.LoginUser;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.okki.linkedin.entity.OkkiLinkedinMarketingData;
import org.jeecg.modules.okki.linkedin.entity.OkkiLinkedinMarketingDataDetail;
import org.jeecg.modules.okki.linkedin.service.IOkkiLinkedinMarketingDataDetailService;
import org.jeecg.modules.okki.linkedin.service.IOkkiLinkedinMarketingDataService;
import org.jeecg.modules.okki.linkedin.vo.OkkiLinkedinMarketingDataPage;
import org.jeecg.modules.okki.youtube.entity.OkkiYoutubeMarketingData;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

 /**
 * @Description: Linkedin营销效果数据
 * @Author: jeecg-boot
 * @Date:   2024-05-16
 * @Version: V1.0
 */
@Api(tags="Linkedin营销效果数据")
@RestController
@RequestMapping("/linkedin/okkiLinkedinMarketingData")
@Slf4j
public class OkkiLinkedinMarketingDataController {
	@Autowired
	private IOkkiLinkedinMarketingDataService okkiLinkedinMarketingDataService;
	@Autowired
	private IOkkiLinkedinMarketingDataDetailService okkiLinkedinMarketingDataDetailService;
	
	/**
	 * 分页列表查询
	 *
	 * @param okkiLinkedinMarketingData
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	//@AutoLog(value = "Linkedin营销效果数据-分页列表查询")
	@ApiOperation(value="Linkedin营销效果数据-分页列表查询", notes="Linkedin营销效果数据-分页列表查询")
	@GetMapping(value = "/list")
	public Result<IPage<OkkiLinkedinMarketingData>> queryPageList(OkkiLinkedinMarketingData okkiLinkedinMarketingData,
								   @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
								   @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
								   HttpServletRequest req) {
		QueryWrapper<OkkiLinkedinMarketingData> queryWrapper = QueryGenerator.initQueryWrapper(okkiLinkedinMarketingData, req.getParameterMap());
		Page<OkkiLinkedinMarketingData> page = new Page<OkkiLinkedinMarketingData>(pageNo, pageSize);
		IPage<OkkiLinkedinMarketingData> pageList = okkiLinkedinMarketingDataService.page(page, queryWrapper);
		return Result.OK(pageList);
	}
	
	/**
	 *   添加
	 *
	 * @param okkiLinkedinMarketingDataPage
	 * @return
	 */
	@AutoLog(value = "Linkedin营销效果数据-添加")
	@ApiOperation(value="Linkedin营销效果数据-添加", notes="Linkedin营销效果数据-添加")
    @RequiresPermissions("linkedin:okki_linkedin_marketing_data:add")
	@PostMapping(value = "/add")
	public Result<String> add(@RequestBody OkkiLinkedinMarketingDataPage okkiLinkedinMarketingDataPage) {
		OkkiLinkedinMarketingData okkiLinkedinMarketingData = new OkkiLinkedinMarketingData();
		BeanUtils.copyProperties(okkiLinkedinMarketingDataPage, okkiLinkedinMarketingData);
		okkiLinkedinMarketingDataService.saveMain(okkiLinkedinMarketingData, okkiLinkedinMarketingDataPage.getOkkiLinkedinMarketingDataDetailList());
		return Result.OK("添加成功！");
	}
	
	/**
	 *  编辑
	 *
	 * @param okkiLinkedinMarketingDataPage
	 * @return
	 */
	@AutoLog(value = "Linkedin营销效果数据-编辑")
	@ApiOperation(value="Linkedin营销效果数据-编辑", notes="Linkedin营销效果数据-编辑")
    @RequiresPermissions("linkedin:okki_linkedin_marketing_data:edit")
	@RequestMapping(value = "/edit", method = {RequestMethod.PUT,RequestMethod.POST})
	public Result<String> edit(@RequestBody OkkiLinkedinMarketingDataPage okkiLinkedinMarketingDataPage) {
		OkkiLinkedinMarketingData okkiLinkedinMarketingData = new OkkiLinkedinMarketingData();
		BeanUtils.copyProperties(okkiLinkedinMarketingDataPage, okkiLinkedinMarketingData);
		OkkiLinkedinMarketingData okkiLinkedinMarketingDataEntity = okkiLinkedinMarketingDataService.getById(okkiLinkedinMarketingData.getId());
		if(okkiLinkedinMarketingDataEntity==null) {
			return Result.error("未找到对应数据");
		}
		okkiLinkedinMarketingDataService.updateMain(okkiLinkedinMarketingData, okkiLinkedinMarketingDataPage.getOkkiLinkedinMarketingDataDetailList());
		return Result.OK("编辑成功!");
	}
	
	/**
	 *   通过id删除
	 *
	 * @param id
	 * @return
	 */
	@AutoLog(value = "Linkedin营销效果数据-通过id删除")
	@ApiOperation(value="Linkedin营销效果数据-通过id删除", notes="Linkedin营销效果数据-通过id删除")
    @RequiresPermissions("linkedin:okki_linkedin_marketing_data:delete")
	@DeleteMapping(value = "/delete")
	public Result<String> delete(@RequestParam(name="id",required=true) String id) {
		okkiLinkedinMarketingDataService.delMain(id);
		return Result.OK("删除成功!");
	}
	
	/**
	 *  批量删除
	 *
	 * @param ids
	 * @return
	 */
	@AutoLog(value = "Linkedin营销效果数据-批量删除")
	@ApiOperation(value="Linkedin营销效果数据-批量删除", notes="Linkedin营销效果数据-批量删除")
    @RequiresPermissions("linkedin:okki_linkedin_marketing_data:deleteBatch")
	@DeleteMapping(value = "/deleteBatch")
	public Result<String> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		this.okkiLinkedinMarketingDataService.delBatchMain(Arrays.asList(ids.split(",")));
		return Result.OK("批量删除成功！");
	}
	
	/**
	 * 通过id查询
	 *
	 * @param id
	 * @return
	 */
	//@AutoLog(value = "Linkedin营销效果数据-通过id查询")
	@ApiOperation(value="Linkedin营销效果数据-通过id查询", notes="Linkedin营销效果数据-通过id查询")
	@GetMapping(value = "/queryById")
	public Result<OkkiLinkedinMarketingData> queryById(@RequestParam(name="id",required=true) String id) {
		OkkiLinkedinMarketingData okkiLinkedinMarketingData = okkiLinkedinMarketingDataService.getById(id);
		if(okkiLinkedinMarketingData==null) {
			return Result.error("未找到对应数据");
		}
		return Result.OK(okkiLinkedinMarketingData);

	}
	
	/**
	 * 通过id查询
	 *
	 * @param id
	 * @return
	 */
	//@AutoLog(value = "Linkedin营销效果数据详情通过主表ID查询")
	@ApiOperation(value="Linkedin营销效果数据详情主表ID查询", notes="Linkedin营销效果数据详情-通主表ID查询")
	@GetMapping(value = "/queryOkkiLinkedinMarketingDataDetailByMainId")
	public Result<List<OkkiLinkedinMarketingDataDetail>> queryOkkiLinkedinMarketingDataDetailListByMainId(@RequestParam(name="id",required=true) String id) {
		List<OkkiLinkedinMarketingDataDetail> okkiLinkedinMarketingDataDetailList = okkiLinkedinMarketingDataDetailService.selectByMainId(id);
		return Result.OK(okkiLinkedinMarketingDataDetailList);
	}

    /**
    * 导出excel
    *
    * @param request
    * @param okkiLinkedinMarketingData
    */
    @RequiresPermissions("linkedin:okki_linkedin_marketing_data:exportXls")
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, OkkiLinkedinMarketingData okkiLinkedinMarketingData) {
      // Step.1 组装查询条件查询数据
      QueryWrapper<OkkiLinkedinMarketingData> queryWrapper = QueryGenerator.initQueryWrapper(okkiLinkedinMarketingData, request.getParameterMap());
      LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();

      //配置选中数据查询条件
       String selections = request.getParameter("selections");
       if(oConvertUtils.isNotEmpty(selections)) {
            List<String> selectionList = Arrays.asList(selections.split(","));
            queryWrapper.in("id",selectionList);
       }
       //Step.2 获取导出数据
       List<OkkiLinkedinMarketingData> okkiLinkedinMarketingDataList = okkiLinkedinMarketingDataService.list(queryWrapper);

      // Step.3 组装pageList
      List<OkkiLinkedinMarketingDataPage> pageList = new ArrayList<OkkiLinkedinMarketingDataPage>();
      for (OkkiLinkedinMarketingData main : okkiLinkedinMarketingDataList) {
          OkkiLinkedinMarketingDataPage vo = new OkkiLinkedinMarketingDataPage();
          BeanUtils.copyProperties(main, vo);
          List<OkkiLinkedinMarketingDataDetail> okkiLinkedinMarketingDataDetailList = okkiLinkedinMarketingDataDetailService.selectByMainId(main.getId());
          vo.setOkkiLinkedinMarketingDataDetailList(okkiLinkedinMarketingDataDetailList);
          pageList.add(vo);
      }

      // Step.4 AutoPoi 导出Excel
      ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
      mv.addObject(NormalExcelConstants.FILE_NAME, "Linkedin营销效果数据列表");
      mv.addObject(NormalExcelConstants.CLASS, OkkiLinkedinMarketingDataPage.class);
      mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("Linkedin营销效果数据数据", "导出人:"+sysUser.getRealname(), "Linkedin营销效果数据"));
      mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
      return mv;
    }

    /**
    * 通过excel导入数据
    *
    * @param request
    * @param response
    * @return
    */
    @RequiresPermissions("linkedin:okki_linkedin_marketing_data:importExcel")
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
      MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
      Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
      for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
          // 获取上传文件对象
          MultipartFile file = entity.getValue();
          ImportParams params = new ImportParams();
          params.setTitleRows(2);
          params.setHeadRows(1);
          params.setNeedSave(true);
          try {
              List<OkkiLinkedinMarketingDataPage> list = ExcelImportUtil.importExcel(file.getInputStream(), OkkiLinkedinMarketingDataPage.class, params);
              for (OkkiLinkedinMarketingDataPage page : list) {
                  OkkiLinkedinMarketingData po = new OkkiLinkedinMarketingData();
                  BeanUtils.copyProperties(page, po);
                  okkiLinkedinMarketingDataService.saveMain(po, page.getOkkiLinkedinMarketingDataDetailList());
              }
              return Result.OK("文件导入成功！数据行数:" + list.size());
          } catch (Exception e) {
              log.error(e.getMessage(),e);
              return Result.error("文件导入失败:"+e.getMessage());
          } finally {
              try {
                  file.getInputStream().close();
              } catch (IOException e) {
                  e.printStackTrace();
              }
          }
      }
      return Result.OK("文件导入失败！");
    }

	 /**
	  * 变更状态
	  * @param okkiLinkedinMarketingData
	  * @return
	  */
	 @PostMapping(value = "/status")
	 public Result<String> changeStatus(@RequestBody OkkiLinkedinMarketingData okkiLinkedinMarketingData) throws Exception {
		 boolean result = okkiLinkedinMarketingDataService.changeStatus(okkiLinkedinMarketingData);
		 if (result) {
			 return Result.OK("操作成功");
		 }else {
			 return Result.error("操作失败");
		 }
	 }
}
