package org.jeecg.modules.okki.websitequality.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.base.controller.JeecgController;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.modules.okki.websitequality.entity.OkkiWebsiteQuality;
import org.jeecg.modules.okki.websitequality.service.IOkkiWebsiteQualityService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;

 /**
 * @Description: 网站质量
 * @Author: jeecg-boot
 * @Date:   2024-05-15
 * @Version: V1.0
 */
@Api(tags="网站质量")
@RestController
@RequestMapping("/websitequality/okkiWebsiteQuality")
@Slf4j
public class OkkiWebsiteQualityController extends JeecgController<OkkiWebsiteQuality, IOkkiWebsiteQualityService> {
	@Autowired
	private IOkkiWebsiteQualityService okkiWebsiteQualityService;
	
	/**
	 * 分页列表查询
	 *
	 * @param okkiWebsiteQuality
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	//@AutoLog(value = "网站质量-分页列表查询")
	@ApiOperation(value="网站质量-分页列表查询", notes="网站质量-分页列表查询")
	@GetMapping(value = "/list")
	public Result<IPage<OkkiWebsiteQuality>> queryPageList(OkkiWebsiteQuality okkiWebsiteQuality,
								   @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
								   @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
								   HttpServletRequest req) {
		QueryWrapper<OkkiWebsiteQuality> queryWrapper = QueryGenerator.initQueryWrapper(okkiWebsiteQuality, req.getParameterMap());
		Page<OkkiWebsiteQuality> page = new Page<OkkiWebsiteQuality>(pageNo, pageSize);
		IPage<OkkiWebsiteQuality> pageList = okkiWebsiteQualityService.page(page, queryWrapper);
		return Result.OK(pageList);
	}
	
	/**
	 *   添加
	 *
	 * @param okkiWebsiteQuality
	 * @return
	 */
	@AutoLog(value = "网站质量-添加")
	@ApiOperation(value="网站质量-添加", notes="网站质量-添加")
	@RequiresPermissions("websitequality:okki_website_quality:add")
	@PostMapping(value = "/add")
	public Result<String> add(@RequestBody OkkiWebsiteQuality okkiWebsiteQuality) {
		okkiWebsiteQualityService.save(okkiWebsiteQuality);
		return Result.OK("添加成功！");
	}
	
	/**
	 *  编辑
	 *
	 * @param okkiWebsiteQuality
	 * @return
	 */
	@AutoLog(value = "网站质量-编辑")
	@ApiOperation(value="网站质量-编辑", notes="网站质量-编辑")
	@RequiresPermissions("websitequality:okki_website_quality:edit")
	@RequestMapping(value = "/edit", method = {RequestMethod.PUT,RequestMethod.POST})
	public Result<String> edit(@RequestBody OkkiWebsiteQuality okkiWebsiteQuality) {
		okkiWebsiteQualityService.updateById(okkiWebsiteQuality);
		return Result.OK("编辑成功!");
	}
	
	/**
	 *   通过id删除
	 *
	 * @param id
	 * @return
	 */
	@AutoLog(value = "网站质量-通过id删除")
	@ApiOperation(value="网站质量-通过id删除", notes="网站质量-通过id删除")
	@RequiresPermissions("websitequality:okki_website_quality:delete")
	@DeleteMapping(value = "/delete")
	public Result<String> delete(@RequestParam(name="id",required=true) String id) {
		okkiWebsiteQualityService.removeById(id);
		return Result.OK("删除成功!");
	}
	
	/**
	 *  批量删除
	 *
	 * @param ids
	 * @return
	 */
	@AutoLog(value = "网站质量-批量删除")
	@ApiOperation(value="网站质量-批量删除", notes="网站质量-批量删除")
	@RequiresPermissions("websitequality:okki_website_quality:deleteBatch")
	@DeleteMapping(value = "/deleteBatch")
	public Result<String> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		this.okkiWebsiteQualityService.removeByIds(Arrays.asList(ids.split(",")));
		return Result.OK("批量删除成功!");
	}
	
	/**
	 * 通过id查询
	 *
	 * @param id
	 * @return
	 */
	//@AutoLog(value = "网站质量-通过id查询")
	@ApiOperation(value="网站质量-通过id查询", notes="网站质量-通过id查询")
	@GetMapping(value = "/queryById")
	public Result<OkkiWebsiteQuality> queryById(@RequestParam(name="id",required=true) String id) {
		OkkiWebsiteQuality okkiWebsiteQuality = okkiWebsiteQualityService.getById(id);
		if(okkiWebsiteQuality==null) {
			return Result.error("未找到对应数据");
		}
		return Result.OK(okkiWebsiteQuality);
	}

    /**
    * 导出excel
    *
    * @param request
    * @param okkiWebsiteQuality
    */
    @RequiresPermissions("websitequality:okki_website_quality:exportXls")
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, OkkiWebsiteQuality okkiWebsiteQuality) {
        return super.exportXls(request, okkiWebsiteQuality, OkkiWebsiteQuality.class, "网站质量");
    }

    /**
      * 通过excel导入数据
    *
    * @param request
    * @param response
    * @return
    */
    @RequiresPermissions("websitequality:okki_website_quality:importExcel")
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        return super.importExcel(request, response, OkkiWebsiteQuality.class);
    }

}
