package org.jeecg.modules.okki.youtube.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.SecurityUtils;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.system.vo.LoginUser;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.okki.youtube.entity.OkkiYoutubeMarketingData;
import org.jeecg.modules.okki.youtube.entity.OkkiYoutubeMarketingDataDetail;
import org.jeecg.modules.okki.youtube.service.IOkkiYoutubeMarketingDataDetailService;
import org.jeecg.modules.okki.youtube.service.IOkkiYoutubeMarketingDataService;
import org.jeecg.modules.okki.youtube.vo.OkkiYoutubeMarketingDataPage;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

 /**
 * @Description: Youtube营销效果数据
 * @Author: jeecg-boot
 * @Date:   2024-05-16
 * @Version: V1.0
 */
@Api(tags="Youtube营销效果数据")
@RestController
@RequestMapping("/youtube/okkiYoutubeMarketingData")
@Slf4j
public class OkkiYoutubeMarketingDataController {
	@Autowired
	private IOkkiYoutubeMarketingDataService okkiYoutubeMarketingDataService;
	@Autowired
	private IOkkiYoutubeMarketingDataDetailService okkiYoutubeMarketingDataDetailService;
	
	/**
	 * 分页列表查询
	 *
	 * @param okkiYoutubeMarketingData
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	//@AutoLog(value = "Youtube营销效果数据-分页列表查询")
	@ApiOperation(value="Youtube营销效果数据-分页列表查询", notes="Youtube营销效果数据-分页列表查询")
	@GetMapping(value = "/list")
	public Result<IPage<OkkiYoutubeMarketingData>> queryPageList(OkkiYoutubeMarketingData okkiYoutubeMarketingData,
								   @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
								   @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
								   HttpServletRequest req) {
		QueryWrapper<OkkiYoutubeMarketingData> queryWrapper = QueryGenerator.initQueryWrapper(okkiYoutubeMarketingData, req.getParameterMap());
		Page<OkkiYoutubeMarketingData> page = new Page<OkkiYoutubeMarketingData>(pageNo, pageSize);
		IPage<OkkiYoutubeMarketingData> pageList = okkiYoutubeMarketingDataService.page(page, queryWrapper);
		return Result.OK(pageList);
	}
	
	/**
	 *   添加
	 *
	 * @param okkiYoutubeMarketingDataPage
	 * @return
	 */
	@AutoLog(value = "Youtube营销效果数据-添加")
	@ApiOperation(value="Youtube营销效果数据-添加", notes="Youtube营销效果数据-添加")
    @RequiresPermissions("youtube:okki_youtube_marketing_data:add")
	@PostMapping(value = "/add")
	public Result<String> add(@RequestBody OkkiYoutubeMarketingDataPage okkiYoutubeMarketingDataPage) {
		OkkiYoutubeMarketingData okkiYoutubeMarketingData = new OkkiYoutubeMarketingData();
		BeanUtils.copyProperties(okkiYoutubeMarketingDataPage, okkiYoutubeMarketingData);
		okkiYoutubeMarketingDataService.saveMain(okkiYoutubeMarketingData, okkiYoutubeMarketingDataPage.getOkkiYoutubeMarketingDataDetailList());
		return Result.OK("添加成功！");
	}
	
	/**
	 *  编辑
	 *
	 * @param okkiYoutubeMarketingDataPage
	 * @return
	 */
	@AutoLog(value = "Youtube营销效果数据-编辑")
	@ApiOperation(value="Youtube营销效果数据-编辑", notes="Youtube营销效果数据-编辑")
    @RequiresPermissions("youtube:okki_youtube_marketing_data:edit")
	@RequestMapping(value = "/edit", method = {RequestMethod.PUT,RequestMethod.POST})
	public Result<String> edit(@RequestBody OkkiYoutubeMarketingDataPage okkiYoutubeMarketingDataPage) {
		OkkiYoutubeMarketingData okkiYoutubeMarketingData = new OkkiYoutubeMarketingData();
		BeanUtils.copyProperties(okkiYoutubeMarketingDataPage, okkiYoutubeMarketingData);
		OkkiYoutubeMarketingData okkiYoutubeMarketingDataEntity = okkiYoutubeMarketingDataService.getById(okkiYoutubeMarketingData.getId());
		if(okkiYoutubeMarketingDataEntity==null) {
			return Result.error("未找到对应数据");
		}
		okkiYoutubeMarketingData.setStatus(0);
		okkiYoutubeMarketingDataService.updateMain(okkiYoutubeMarketingData, okkiYoutubeMarketingDataPage.getOkkiYoutubeMarketingDataDetailList());
		return Result.OK("编辑成功!");
	}
	
	/**
	 *   通过id删除
	 *
	 * @param id
	 * @return
	 */
	@AutoLog(value = "Youtube营销效果数据-通过id删除")
	@ApiOperation(value="Youtube营销效果数据-通过id删除", notes="Youtube营销效果数据-通过id删除")
    @RequiresPermissions("youtube:okki_youtube_marketing_data:delete")
	@DeleteMapping(value = "/delete")
	public Result<String> delete(@RequestParam(name="id",required=true) String id) {
		okkiYoutubeMarketingDataService.delMain(id);
		return Result.OK("删除成功!");
	}
	
	/**
	 *  批量删除
	 *
	 * @param ids
	 * @return
	 */
	@AutoLog(value = "Youtube营销效果数据-批量删除")
	@ApiOperation(value="Youtube营销效果数据-批量删除", notes="Youtube营销效果数据-批量删除")
    @RequiresPermissions("youtube:okki_youtube_marketing_data:deleteBatch")
	@DeleteMapping(value = "/deleteBatch")
	public Result<String> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		this.okkiYoutubeMarketingDataService.delBatchMain(Arrays.asList(ids.split(",")));
		return Result.OK("批量删除成功！");
	}
	
	/**
	 * 通过id查询
	 *
	 * @param id
	 * @return
	 */
	//@AutoLog(value = "Youtube营销效果数据-通过id查询")
	@ApiOperation(value="Youtube营销效果数据-通过id查询", notes="Youtube营销效果数据-通过id查询")
	@GetMapping(value = "/queryById")
	public Result<OkkiYoutubeMarketingData> queryById(@RequestParam(name="id",required=true) String id) {
		OkkiYoutubeMarketingData okkiYoutubeMarketingData = okkiYoutubeMarketingDataService.getById(id);
		if(okkiYoutubeMarketingData==null) {
			return Result.error("未找到对应数据");
		}
		return Result.OK(okkiYoutubeMarketingData);

	}
	
	/**
	 * 通过id查询
	 *
	 * @param id
	 * @return
	 */
	//@AutoLog(value = "Youtube营销效果数据详情-通过主表ID查询")
	@ApiOperation(value="Youtube营销效果数据详情-通过主表ID查询", notes="Youtube营销效果数据详情-通过主表ID查询")
	@GetMapping(value = "/queryOkkiYoutubeMarketingDataDetailByMainId")
	public Result<IPage<OkkiYoutubeMarketingDataDetail>> queryOkkiYoutubeMarketingDataDetailListByMainId(@RequestParam(name="id",required=true) String id) {
		List<OkkiYoutubeMarketingDataDetail> okkiYoutubeMarketingDataDetailList = okkiYoutubeMarketingDataDetailService.selectByMainId(id);
		IPage <OkkiYoutubeMarketingDataDetail> page = new Page<>();
		page.setRecords(okkiYoutubeMarketingDataDetailList);
		page.setTotal(okkiYoutubeMarketingDataDetailList.size());
		return Result.OK(page);
	}

    /**
    * 导出excel
    *
    * @param request
    * @param okkiYoutubeMarketingData
    */
    @RequiresPermissions("youtube:okki_youtube_marketing_data:exportXls")
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, OkkiYoutubeMarketingData okkiYoutubeMarketingData) {
      // Step.1 组装查询条件查询数据
      QueryWrapper<OkkiYoutubeMarketingData> queryWrapper = QueryGenerator.initQueryWrapper(okkiYoutubeMarketingData, request.getParameterMap());
      LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();

     //配置选中数据查询条件
      String selections = request.getParameter("selections");
      if(oConvertUtils.isNotEmpty(selections)) {
           List<String> selectionList = Arrays.asList(selections.split(","));
           queryWrapper.in("id",selectionList);
      }
      //Step.2 获取导出数据
      List<OkkiYoutubeMarketingData>  okkiYoutubeMarketingDataList = okkiYoutubeMarketingDataService.list(queryWrapper);

      // Step.3 组装pageList
      List<OkkiYoutubeMarketingDataPage> pageList = new ArrayList<OkkiYoutubeMarketingDataPage>();
      for (OkkiYoutubeMarketingData main : okkiYoutubeMarketingDataList) {
          OkkiYoutubeMarketingDataPage vo = new OkkiYoutubeMarketingDataPage();
          BeanUtils.copyProperties(main, vo);
          List<OkkiYoutubeMarketingDataDetail> okkiYoutubeMarketingDataDetailList = okkiYoutubeMarketingDataDetailService.selectByMainId(main.getId());
          vo.setOkkiYoutubeMarketingDataDetailList(okkiYoutubeMarketingDataDetailList);
          pageList.add(vo);
      }

      // Step.4 AutoPoi 导出Excel
      ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
      mv.addObject(NormalExcelConstants.FILE_NAME, "Youtube营销效果数据列表");
      mv.addObject(NormalExcelConstants.CLASS, OkkiYoutubeMarketingDataPage.class);
      mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("Youtube营销效果数据数据", "导出人:"+sysUser.getRealname(), "Youtube营销效果数据"));
      mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
      return mv;
    }

    /**
    * 通过excel导入数据
    *
    * @param request
    * @param response
    * @return
    */
    @RequiresPermissions("youtube:okki_youtube_marketing_data:importExcel")
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
      MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
      Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
      for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
          // 获取上传文件对象
          MultipartFile file = entity.getValue();
          ImportParams params = new ImportParams();
          params.setTitleRows(2);
          params.setHeadRows(1);
          params.setNeedSave(true);
          try {
              List<OkkiYoutubeMarketingDataPage> list = ExcelImportUtil.importExcel(file.getInputStream(), OkkiYoutubeMarketingDataPage.class, params);
              for (OkkiYoutubeMarketingDataPage page : list) {
                  OkkiYoutubeMarketingData po = new OkkiYoutubeMarketingData();
                  BeanUtils.copyProperties(page, po);
                  okkiYoutubeMarketingDataService.saveMain(po, page.getOkkiYoutubeMarketingDataDetailList());
              }
              return Result.OK("文件导入成功！数据行数:" + list.size());
          } catch (Exception e) {
              log.error(e.getMessage(),e);
              return Result.error("文件导入失败:"+e.getMessage());
          } finally {
              try {
                  file.getInputStream().close();
              } catch (IOException e) {
                  e.printStackTrace();
              }
          }
      }
      return Result.OK("文件导入失败！");
    }

	 /**
	  * 变更状态
	  * @param okkiYoutubeMarketingData
	  * @return
	  */
	 @PostMapping(value = "/status")
	 public Result<String> changeStatus(@RequestBody OkkiYoutubeMarketingData okkiYoutubeMarketingData) throws Exception {
		 boolean result = okkiYoutubeMarketingDataService.changeStatus(okkiYoutubeMarketingData);
		 if (result) {
			 return Result.OK("操作成功");
		 }else {
			 return Result.error("操作失败");
		 }
	 }
}
